###########################################################################
#                                                                         #
# File   :	gv_helpbrowser.tcl                                        #
# Project:	TkMount                                                   #
#                                                                         #
# Started:	22.09.97                                                  #
# Changed:	22.09.97                                                  #
# Author :      Gerhard Vogt                                              #
#                                                                         #
# Description:	This file contains the hepbrowser procedure.              #
#                                                                         #
# Copyright (C) 1997 by Gerhard Vogt                                      #
#                                                                         #
# This program is free software; you can redistribute it and/or modify    #
# it under the terms of the GNU General Public License as published by    #
# the Free Software Foundation; either version 2 of the License, or       #
# (at your option) any later version.                                     #
#                                                                         #
# This program is distributed in the hope that it will be useful,         #
# but WITHOUT ANY WARRANTY; without even the implied warranty of          #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           #
# GNU General Public License for more details.                            #
#                                                                         #
# You should have received a copy of the GNU General Public License       #
# along with this program; if not, write to the Free Software             #
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.               #
# See the file "COPYING" in the base directory of this distribution       #
# for more.                                                               #
#                                                                         #
###########################################################################

###########################################################################
# initialise some variables:                                              #
###########################################################################

set text ""
set labinfo ""
set yesno "no"
set altstart 1.0
set tags(none) ""
set maintopics 0
set topics ""

proc gv_searchMatches {w pattern script} {
  scan [$w index end] %d numLines
  for {set i 1} {$i < $numLines} {incr i} {
    $w mark set last $i.0
    while {[regexp -indices $pattern [$w get last "last lineend"] indices]} {
      $w mark set first "last + [lindex $indices 0] chars"
      $w mark set last "last + 1 chars + [lindex $indices 1] chars"
      uplevel $script
      }
    }
  }
  
  proc gv_endofText {} {
    global yesno
    catch {destroy .eotw}
    toplevel .eotw -class Dialog
    wm withdraw .eotw
    wm title .eotw "Start at Top?"
    frame .eotw.main
    frame .eotw.bottom
    message .eotw.main.msg -text "You have reached the end of the text. Do you want to start at top?" -width 12c
    pack .eotw.main .eotw.bottom
    pack .eotw.main.msg -in .eotw.main
    button .eotw.bottom.yes -text "Yes" -command {
      set yesno yes
      destroy .eotw
      set start 1.0
      .hw.top.right.t1 yview 1.0
      catch {set index [.hw.top.right.t1 search -nocase -count count $searchvar $start]}
      .hw.top.right.t1 yview $index
      }
    button .eotw.bottom.no -text "No" -command {set yesno no; destroy .eotw}
    pack .eotw.bottom.yes .eotw.bottom.no -in .eotw.bottom -side left
    wm withdraw .eotw
    update idletasks
    set x [expr [winfo screenwidth .eotw]/2 -[winfo reqwidth .eotw]/2 - [winfo vrootx [winfo parent .eotw]]]
    set y [expr [winfo screenheight .eotw]/2 -[winfo reqheight .eotw]/2 - [winfo vrooty [winfo parent .eotw]]]
    wm geom .eotw +$x+$y
    wm deiconify .eotw
    grab set .eotw
  }

  proc gv_endofText2 {} {
    global yesno
    catch {destroy .eotw}
    toplevel .eotw -class Dialog
    wm withdraw .eotw
    wm title .eotw "Start at Top?"
    frame .eotw.main
    frame .eotw.bottom
    message .eotw.main.msg -text "You have reached the end of the text. Do you want to start at top?" -width 12c
    pack .eotw.main .eotw.bottom
    pack .eotw.main.msg -in .eotw.main
    button .eotw.bottom.yes -text "Yes" -command {
      set yesno yes
      destroy .eotw
      set start 1.0
      .licw.top.t1 yview 1.0
      catch {set index [.licw.top.t1 search -nocase -count count $searchvar $start]}
      .licw.top.t1 yview $index
      }
    button .eotw.bottom.no -text "No" -command {set yesno no; destroy .eotw}
    pack .eotw.bottom.yes .eotw.bottom.no -in .eotw.bottom -side left
    wm withdraw .eotw
    update idletasks
    set x [expr [winfo screenwidth .eotw]/2 -[winfo reqwidth .eotw]/2 - [winfo vrootx [winfo parent .eotw]]]
    set y [expr [winfo screenheight .eotw]/2 -[winfo reqheight .eotw]/2 - [winfo vrooty [winfo parent .eotw]]]
    wm geom .eotw +$x+$y
    wm deiconify .eotw
    grab set .eotw    
  }

  proc gv_notfound {} {
    global yesno
    catch {destroy .nfw}
    toplevel .nfw -class Dialog
    wm withdraw .nfw
    wm title .nfw "Start at Top?"
    frame .nfw.main
    frame .nfw.bottom
    message .nfw.main.msg -text "Search pattern not found!" -width 12c
    pack .nfw.main .nfw.bottom
    pack .nfw.main.msg -in .nfw.main
    button .nfw.bottom.yes -text "OK" -command {
      destroy .nfw}
    pack .nfw.bottom.yes -in .nfw.bottom -side left
    wm withdraw .nfw
    update idletasks
    set x [expr [winfo screenwidth .nfw]/2 -[winfo reqwidth .nfw]/2 - [winfo vrootx [winfo parent .nfw]]]
    set y [expr [winfo screenheight .nfw]/2 -[winfo reqheight .nfw]/2 - [winfo vrooty [winfo parent .nfw]]]
    wm geom .nfw +$x+$y
    wm deiconify .nfw
    grab set .nfw    
  }

###########################################################################
# Helpbrowser main routine:                                               #
###########################################################################

proc gv_helpbrowser {} {

global yesno labinfo text altstart maintopics topics srcpath

upvar tags tags

  catch {destroy .hw}
  toplevel .hw -class Dialog
  wm title .hw Help

# frame .hw -height 15c -width 20c
# pack propagate .hw 0
# pack .hw
frame .hw.top
frame .hw.top.right
frame .hw.top.left
frame .hw.bottom

pack .hw.top .hw.bottom -in .hw -side top
pack .hw.top.left .hw.top.right -in .hw.top -side left

###########################################################################
# Define Listbox:                                                         #
###########################################################################

listbox .hw.top.left.content -relief sunken -borderwidth 2 -yscrollcommand ".hw.top.left.scroll set" -height 15 -width 30
scrollbar .hw.top.left.scroll -command ".hw.top.left.content yview"

pack .hw.top.left.content .hw.top.left.scroll -in .hw.top.left -fill y -side left

###########################################################################
# Bind Listbox:                                                           #
###########################################################################

 bind .hw.top.left.content <Button-1> {
  after 100 {
  .hw.top.right.t1 yview $tags([selection get])
  }
  bind .hw.top.left.content <Double-Button-1> {
    after 100 {

      catch {set sel [selection get]}
      for {set maint 0} {$maint < $maintopics} {incr maint} {
        set s [lindex $topics $maint]
        if {$sel == [lindex $s 1]} {
          
          if {[lindex $s 0] == "+"} {                
            set s [lreplace $s 0 0 "-"]
            } else {
            set s [lreplace $s 0 0 "+"]
            }
          set topics [lreplace $topics $maint $maint $s]
          }
        }
        
    .hw.top.left.content delete 0 end
    for {set maint 0} {$maint < $maintopics} {incr maint} {
      set s [lindex $topics $maint]
      set labinfo [llength $s]
      if {[lindex $s 0]=="+"} {
        for {set submaint 1} {$submaint < [llength $s]} {incr submaint} {
          .hw.top.left.content insert end [lindex $s $submaint]}
        } else {
          .hw.top.left.content insert end [lindex $s 1]}
        }
      }  
    }
  }

###########################################################################
# Define Buttons/Labels/Entrys:                                           #
###########################################################################

label .hw.bottom.lab -text Searchpattern:
# label .hw.bottom.lab -textvariable labinfo
button .hw.bottom.ok -text OK -command {destroy .hw}
entry .hw.bottom.sentry -width 30 -relief sunken -bd 2 -textvariable searchvar
button .hw.bottom.sbutton -text Search -command {
  if {$searchvar != ""} {        
.hw.top.right.t1 tag delete found        
  set totalNoOfLines [lindex [.hw.top.right.t1 index end] 0]
  set indexStartOfPage [lindex [.hw.top.right.t1 yview] 0]
  set start [.hw.top.right.t1 index \
   "[expr int($totalNoOfLines * $indexStartOfPage)].0 + 2l"]
  .hw.top.right.t1 yview $start
 gv_searchMatches .hw.top.right.t1 $searchvar {
    .hw.top.right.t1 tag add found first last}
    .hw.top.right.t1 tag  configure found -background red

    set index [.hw.top.right.t1 search -nocase -count count $searchvar $start]
    set error [catch {.hw.top.right.t1 yview $index}]
    if {$error==1} {gv_notfound}
    set labinfo $altstart
 if {[.hw.top.right.t1 yview]==$altstart} {
       gv_endofText
}
    set altstart [.hw.top.right.t1 yview]
    }}

pack .hw.bottom.ok .hw.bottom.lab .hw.bottom.sentry .hw.bottom.sbutton -in .hw.bottom -side left

###########################################################################
# Define Textfield:                                                       #
###########################################################################

  text .hw.top.right.t1 -height 20 -width 70 -yscrollcommand ".hw.top.right.scroll set" -wrap word -cursor arrow 
  scrollbar .hw.top.right.scroll -command ".hw.top.right.t1 yview"
  pack .hw.top.right.t1 .hw.top.right.scroll -in .hw.top.right -side left -fill y

# Fill Texfield
  cd $srcpath
  set Datei [open "help.html" r]
  while {[gets $Datei Zeile] >= 0} {
  .hw.top.right.t1 insert end $Zeile\n}
  close $Datei

 set indexhl 0
 
###########################################################################
# Headlines:                                                              #
###########################################################################

 set maintopics 0
 gv_searchMatches .hw.top.right.t1 {(<h[0-9]>)([a-z," ",A-Z,:,-,_,-]*)(</h[0-9]>)} {
   .hw.top.right.t1 tag add hline$indexhl first last
   set indices [.hw.top.right.t1 tag ranges hline$indexhl]   
   
   set headline [.hw.top.right.t1 get [lindex $indices 0] [lindex $indices 1]]
   set number $headline
   regsub {([a-z," ",A-Z,:,-,_,-]*)(</h[0-9]>)} $number {} number
   regsub {<h} $number {} number
   regsub {>} $number {} number

   regsub {<h[0-9]>} $headline {} headline
   regsub {</h[0-9]>} $headline {} headline
   if {$number == 1} {
     .hw.top.right.t1 tag add mainhline first last
     }
   switch $number {
      1 {set headline "$headline"}
      2 {set headline "    $headline"}
      3 {set headline "        $headline"}
      4 {set headline "            $headline"}
      5 {set headline "                $headline"}
      6 {set headline "                    $headline"}
      7 {set headline "                        $headline"}
      }
      
   set tags($headline) [lindex $indices 0]

# Fill Listbox:
   if {$number == 1} {
     incr maintopics
     lappend maintopics$maintopics - $headline
     } else {
     lappend maintopics$maintopics $headline
     }
# .hw.top.left.content insert end $headline
   incr indexhl
   }
  for {set maint 1} {$maint <= $maintopics} {incr maint} {
    eval set s "\$maintopics$maint"
    lappend topics $s
    }
  for {set maint 0} {$maint < $maintopics} {incr maint} {
    set s [lindex $topics $maint]
    set labinfo [llength $s]
    if {[lindex $s 0]=="+"} {
    for {set submaint 1} {$submaint < [llength $s]} {incr submaint} {
      .hw.top.left.content insert end [lindex $s $submaint]}
      } else {
      .hw.top.left.content insert end [lindex $s 1]}
    }

  for {set index 0} {$index <= $indexhl} {incr index} {
    .hw.top.right.t1 tag configure hline$index \
    -font -adobe-courier-Bold-I-Normal-*-*-150-*-*-*-*-*-*
    }

    .hw.top.right.t1 tag configure mainhline \
    -font -adobe-courier-Bold-I-Normal-*-*-300-*-*-*-*-*-*

###########################################################################
# Deleting <>                                                             #
###########################################################################
  gv_searchMatches .hw.top.right.t1 {<h[0-9]>} {
    .hw.top.right.t1 delete first last}
  gv_searchMatches .hw.top.right.t1 {</h[0-9]>} {
    .hw.top.right.t1 delete first last}

.hw.top.right.t1 configure -state disabled
  grab set .hw
  wm withdraw .hw
  update idletasks
  set x [expr [winfo screenwidth .hw]/2 -[winfo reqwidth .hw]/2 - [winfo vrootx [winfo parent .hw]]]
  set y [expr [winfo screenheight .hw]/2 -[winfo reqheight .hw]/2 - [winfo vrooty [winfo parent .hw]]]
  wm geom .hw +$x+$y
  wm deiconify .hw
  tkwait window .hw
}

###########################################################################
# simple help-browser:                                                    #
###########################################################################

proc gv_licensebrowser {file windowname} {

global yesno labinfo text altstart srcpath
upvar tags tags

  catch {destroy .licw}
  toplevel .licw -class Dialog
  wm title .licw $windowname

  frame .licw.top
  frame .licw.bottom

  pack .licw.top .licw.bottom -in .licw -side top

###########################################################################
# Define Textfield:                                                       #
###########################################################################

  text .licw.top.t1 -height 20 -width 80 -yscrollcommand ".licw.top.scroll set" -wrap word -cursor arrow 
  scrollbar .licw.top.scroll -command ".licw.top.t1 yview"
  pack .licw.top.t1 .licw.top.scroll -in .licw.top -side left -fill y
  
###########################################################################
# Fill Texfield                                                           #
###########################################################################

  cd $srcpath
  set Datei [open $file r]
  while {[gets $Datei Zeile] >= 0} {
  .licw.top.t1 insert end $Zeile\n}
  close $Datei
  
###########################################################################
# Define Buttons/Labels/Entrys:                                           #
###########################################################################

label .licw.bottom.lab -text Searchpattern:

button .licw.bottom.ok -text OK -command {destroy .licw}
entry .licw.bottom.sentry -width 30 -relief sunken -bd 2 -textvariable searchvar
button .licw.bottom.sbutton -text Search -command {
  if {$searchvar != ""} {
.licw.top.t1 tag delete found
  set totalNoOfLines [lindex [.licw.top.t1 index end] 0]
  set indexStartOfPage [lindex [.licw.top.t1 yview] 0]
  set start [.licw.top.t1 index \
   "[expr int($totalNoOfLines * $indexStartOfPage)].0 + 2l"]
  .licw.top.t1 yview $start
 gv_searchMatches .licw.top.t1 $searchvar {
    .licw.top.t1 tag add found first last}
    .licw.top.t1 tag  configure found -background red

    set index [.licw.top.t1 search -nocase -count count $searchvar $start]
    set error [catch {.licw.top.t1 yview $index}]
    if {$error==1} {gv_notfound}    
    set labinfo $altstart
 if {[.licw.top.t1 yview]==$altstart} {
       gv_endofText2
       if {$yesno == "yes"} {}
}
    set altstart [.licw.top.t1 yview]

    }}
pack .licw.bottom.ok .licw.bottom.lab .licw.bottom.sentry .licw.bottom.sbutton -in .licw.bottom -side left  

.licw.top.t1 configure -state disabled

  wm withdraw .licw
  update idletasks
  set x [expr [winfo screenwidth .licw]/2 -[winfo reqwidth .licw]/2 - [winfo vrootx [winfo parent .licw]]]
  set y [expr [winfo screenheight .licw]/2 -[winfo reqheight .licw]/2 - [winfo vrooty [winfo parent .licw]]]
  wm geom .licw +$x+$y
  wm deiconify .licw
  grab set .licw
}
